@@ -81,7 +81,7 @@ enum
 
 static guint session_cfg_signals [LAST_SIGNAL] = { 0 };
 
-G_DEFINE_TYPE (BraseroSessionCfg, brasero_session_cfg, BRASERO_TYPE_BURN_SESSION);
+G_DEFINE_TYPE (BraseroSessionCfg, brasero_session_cfg, BRASERO_TYPE_SESSION_SPAN);
 
 #define BRASERO_DEST_SAVED_FLAGS		(BRASERO_DRIVE_PROPERTIES_FLAGS|BRASERO_BURN_FLAG_MULTI)
 #define BRASERO_DRIVE_PROPERTIES_KEY		"/apps/brasero/drives"
@@ -264,7 +264,6 @@ brasero_session_cfg_add_drive_properties_flags (BraseroSessionCfg *self,
 
 	/* These are always supported and better be set. */
 	brasero_burn_session_set_flags (BRASERO_BURN_SESSION (self),
-					BRASERO_BURN_FLAG_DONT_OVERWRITE|
 					BRASERO_BURN_FLAG_CHECK_SIZE|
 					BRASERO_BURN_FLAG_NOGRACE);
 
@@ -480,7 +479,6 @@ brasero_session_cfg_check_size (BraseroSessionCfg *self)
 	gint64 session_size;
 	gint64 max_sectors;
 	gint64 disc_size;
-	GSList *iter;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
 
@@ -505,14 +503,15 @@ brasero_session_cfg_check_size (BraseroSessionCfg *self)
 	flags = brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self));
 	if (flags & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND))
 		brasero_medium_get_free_space (medium, NULL, &disc_size);
-	else
+	else if (brasero_burn_session_can_blank (BRASERO_BURN_SESSION (self)) == BRASERO_BURN_OK)
 		brasero_medium_get_capacity (medium, NULL, &disc_size);
+	else
+		brasero_medium_get_free_space (medium, NULL, &disc_size);
 
 	if (disc_size < 0)
 		disc_size = 0;
 
 	/* get input track size */
-	iter = brasero_burn_session_get_tracks (BRASERO_BURN_SESSION (self));
 	session_size = 0;
 
 	if (brasero_burn_session_tag_lookup (BRASERO_BURN_SESSION (self),
@@ -525,17 +524,10 @@ brasero_session_cfg_check_size (BraseroSessionCfg *self)
 						  &value) == BRASERO_BURN_OK) {
 		session_size = g_value_get_int64 (value);
 	}
-	else for (; iter; iter = iter->next) {
-		BraseroTrack *track;
-
-		track = iter->data;
-		if (!BRASERO_IS_TRACK_DATA (track)) {
-			goffset sectors = 0;
-
-			brasero_track_get_size (track, &sectors, NULL);
-			session_size += sectors;
-		}
-	}
+	else
+		brasero_burn_session_get_size (BRASERO_BURN_SESSION (self),
+					       &session_size,
+					       NULL);
 
 	BRASERO_BURN_LOG ("Session size %lli/Disc size %lli",
 			  session_size,
