@@ -34,7 +34,7 @@
 
 #include "brasero-project-parse.h"
 #include "burn-basics.h"
-#include "burn-session.h"
+#include "brasero-session.h"
 
 G_BEGIN_DECLS
 
@@ -89,8 +89,6 @@ struct _BraseroDiscIface {
 	BraseroDiscResult	(*get_track)		(BraseroDisc *disc,
 							 BraseroDiscTrack *track);
 
-	BraseroDiscResult	(*set_session_param)	(BraseroDisc *disc,
-							 BraseroBurnSession *session);
 	BraseroDiscResult	(*set_session_contents)	(BraseroDisc *disc,
 							 BraseroBurnSession *session);
 
@@ -150,9 +148,6 @@ brasero_disc_load_track (BraseroDisc *disc,
 			 BraseroDiscTrack *track);
 
 BraseroDiscResult
-brasero_disc_set_session_param (BraseroDisc *disc,
-				BraseroBurnSession *session);
-BraseroDiscResult
 brasero_disc_set_session_contents (BraseroDisc *disc,
 				   BraseroBurnSession *session);
 
