@@ -46,28 +46,34 @@
 
 #include <gconf/gconf-client.h>
 
+#include <gst/gst.h>
+
 #ifdef BUILD_PLAYLIST
 #include <totem-pl-parser.h>
 #endif
 
-#include "burn-debug.h"
-#include "burn-session.h"
+#include "brasero-misc.h"
+#include "brasero-jacket-edit.h"
+
+#include "brasero-tags.h"
+#include "brasero-session.h"
 
 #ifdef BUILD_PREVIEW
 #include "brasero-player.h"
 #endif
 
+#include "brasero-track-data.h"
+#include "brasero-session-cfg.h"
+#include "brasero-burn-options.h"
+#include "brasero-cover.h"
+
+#include "brasero-project-type-chooser.h"
 #include "brasero-app.h"
 #include "brasero-project.h"
-#include "brasero-session-cfg.h"
-#include "brasero-jacket-edit.h"
-#include "brasero-project-type-chooser.h"
 #include "brasero-disc.h"
 #include "brasero-data-disc.h"
 #include "brasero-audio-disc.h"
 #include "brasero-video-disc.h"
-#include "brasero-disc-option-dialog.h"
-#include "brasero-utils.h"
 #include "brasero-uri-container.h"
 #include "brasero-layout-object.h"
 #include "brasero-disc-message.h"
@@ -148,6 +154,8 @@ typedef enum {
 
 struct BraseroProjectPrivate {
 	GtkWidget *name_display;
+	GtkWidget *button_img;
+	GtkWidget *icon_img;
 	GtkWidget *discs;
 	GtkWidget *audio;
 	GtkWidget *data;
@@ -410,11 +418,122 @@ brasero_project_name_changed_cb (BraseroProjectName *name,
 		gtk_action_set_sensitive (action, FALSE);
 }
 
+static void
+brasero_project_data_icon_error (BraseroProject *project,
+				 GError *error)
+{
+	if (error) {
+		brasero_app_alert (brasero_app_get_default (),
+				   /* Translators: this is a picture not
+				    * a disc image */
+				   C_("picture", "Please select another image."),
+				   error->message,
+				   GTK_MESSAGE_ERROR);
+	}
+	else {
+		brasero_app_alert (brasero_app_get_default (),
+				   /* Translators: this is a picture not
+				    * a disc image */
+				   C_("picture", "Please select another image."),
+				   _("Unknown error"),
+				   GTK_MESSAGE_ERROR);
+	}
+}
+
+static void
+brasero_project_icon_changed_cb (BraseroDisc *disc,
+				 BraseroProject *project)
+{
+	GError *error = NULL;
+	GdkPixbuf *pixbuf;
+	gchar *icon; 
+
+	icon = brasero_data_disc_get_scaled_icon_path (BRASERO_DATA_DISC (project->priv->current));
+	if (!icon) {
+		gtk_image_set_from_icon_name (GTK_IMAGE (project->priv->icon_img),
+					      "media-optical",
+					      GTK_ICON_SIZE_LARGE_TOOLBAR);
+		return;
+	}
+
+	/* Load and convert (48x48) the image into a pixbuf */
+	pixbuf = gdk_pixbuf_new_from_file_at_scale (icon,
+						    48,
+						    48,
+						    FALSE,
+						    &error);
+	g_free (icon);
+
+	if (!pixbuf) {
+		gtk_image_set_from_icon_name (GTK_IMAGE (project->priv->icon_img),
+					      "media-optical",
+					      GTK_ICON_SIZE_LARGE_TOOLBAR);
+		brasero_project_data_icon_error (project, error);
+		g_error_free (error);
+		return;
+	}
+
+	gtk_image_set_from_pixbuf (GTK_IMAGE (project->priv->icon_img), pixbuf);
+	g_object_unref (pixbuf);
+}
+
+static void
+brasero_project_icon_button_clicked (GtkWidget *button,
+				     BraseroProject *project)
+{
+	GtkFileFilter *filter;
+	GError *error = NULL;
+	GtkWidget *chooser;
+	gchar *path;
+	gint res;
+
+	if (!BRASERO_IS_DATA_DISC (project->priv->current))
+		return;
+
+	chooser = gtk_file_chooser_dialog_new (_("Medium Icon"),
+					       GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (project))),
+					       GTK_FILE_CHOOSER_ACTION_OPEN,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       GTK_STOCK_OK, GTK_RESPONSE_OK,
+					       NULL);
+
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter, _("All files"));
+	gtk_file_filter_add_pattern (filter, "*");
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), filter);
+
+	filter = gtk_file_filter_new ();
+	/* Translators: this is an image, a picture, not a "Disc Image" */
+	gtk_file_filter_set_name (filter, C_("picture", "Image files"));
+	gtk_file_filter_add_mime_type (filter, "image/*");
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), filter);
+
+	gtk_widget_show (chooser);
+	res = gtk_dialog_run (GTK_DIALOG (chooser));
+	if (res != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (chooser);
+		return;
+	}
+
+	path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+	gtk_widget_destroy (chooser);
+
+	if (!brasero_data_disc_set_icon_path (BRASERO_DATA_DISC (project->priv->current), path, &error)) {
+		if (error) {
+			brasero_project_data_icon_error (project, error);
+			g_error_free (error);
+		}
+	}
+	g_free (path);
+}
+
 static void
 brasero_project_init (BraseroProject *obj)
 {
 	GtkSizeGroup *size_group;
 	GtkWidget *alignment;
+	GtkWidget *button;
+	GtkWidget *image;
 	GtkWidget *label;
 	GtkWidget *box;
 
@@ -435,6 +554,25 @@ brasero_project_init (BraseroProject *obj)
 	gtk_widget_show (box);
 	gtk_box_pack_end (GTK_BOX (obj), box, FALSE, TRUE, 0);
 
+	/* Icon button */
+	image = gtk_image_new_from_icon_name ("media-optical", GTK_ICON_SIZE_LARGE_TOOLBAR);
+	gtk_widget_show (image);
+	obj->priv->icon_img = image;
+
+	button = gtk_button_new ();
+	gtk_widget_show (button);
+	gtk_button_set_image (GTK_BUTTON (button), image);
+	obj->priv->button_img = button;
+
+	gtk_widget_set_tooltip_text (button, _("Select an icon for the disc that will appear in file managers"));
+
+	g_signal_connect (button,
+			  "clicked",
+			  G_CALLBACK (brasero_project_icon_button_clicked),
+			  obj);
+
+	gtk_box_pack_start (GTK_BOX (box), button, FALSE, TRUE, 0);
+
 	/* Name widget */
 	label = gtk_label_new_with_mnemonic (_("_Name:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -511,6 +649,10 @@ brasero_project_init (BraseroProject *obj)
 			  "selection-changed",
 			  G_CALLBACK (brasero_project_selection_changed_cb),
 			  obj);
+	g_signal_connect (obj->priv->data,
+			  "icon-changed",
+			  G_CALLBACK (brasero_project_icon_changed_cb),
+			  obj);
 
 	obj->priv->video = brasero_video_disc_new ();
 	gtk_widget_show (obj->priv->video);
@@ -576,6 +718,22 @@ brasero_project_new ()
 }
 
 /********************************** size ***************************************/
+gchar *
+brasero_project_get_sectors_string (gint64 sectors,
+				    gboolean time_format)
+{
+	gint64 size_bytes;
+
+	if (time_format) {
+		size_bytes = sectors * GST_SECOND / 75;
+		return brasero_units_get_time_string (size_bytes, TRUE, FALSE);
+	}
+	else {
+		size_bytes = sectors * 2048;
+		return g_format_size_for_display (size_bytes);
+	}
+}
+
 static void
 brasero_project_update_project_size (BraseroProject *project,
 				     guint64 sectors)
@@ -592,10 +750,8 @@ brasero_project_update_project_size (BraseroProject *project,
 
 	gtk_statusbar_pop (GTK_STATUSBAR (status), project->priv->status_ctx);
 
-	string = brasero_utils_get_sectors_string (sectors,
-						   !BRASERO_IS_DATA_DISC (project->priv->current),
-						   TRUE,
-						   FALSE);
+	string = brasero_project_get_sectors_string (sectors,
+						     !BRASERO_IS_DATA_DISC (project->priv->current));
 	if (project->priv->merging) {
 		gchar *medium_string;
 		BraseroMedium *medium;
@@ -741,10 +897,8 @@ _wait_for_ready_state (GtkWidget *dialog)
 			g_free (string);
 		}
 
-		string = brasero_utils_get_sectors_string (project->priv->sectors,
-							   !BRASERO_IS_DATA_DISC (project->priv->current),
-							   TRUE,
-							   FALSE);
+		string = brasero_project_get_sectors_string (project->priv->sectors,
+							     !BRASERO_IS_DATA_DISC (project->priv->current));
 
 		size_str = g_strdup_printf (_("Project estimated size: %s"), string);
 		g_free (string);
@@ -780,12 +934,10 @@ brasero_project_check_status (BraseroProject *project,
 
 	/* we are not ready to create tracks presumably because
 	 * data or audio has not finished to explore a directory
-	 * or get the metadata of a song or a film
-	 */
+	 * or get the metadata of a song or a film  */
 
 	/* This dialog can run as a standalone window when run from nautilus
-	 * to burn burn:// URI contents.
-	 */
+	 * to burn burn:// URI contents. */
 	dialog = brasero_app_dialog (brasero_app_get_default (),
 				     _("Please wait until the estimation of the project size is completed."),
 				     GTK_BUTTONS_CANCEL,
@@ -862,26 +1014,6 @@ brasero_project_check_status (BraseroProject *project,
 	return brasero_disc_get_status (disc, NULL, NULL);
 }
 
-/******************************** cover ****************************************/
-void
-brasero_project_set_cover_specifics (BraseroProject *self,
-				     BraseroJacketEdit *cover)
-{
-	BraseroBurnSession *session;
-
-	if (!BRASERO_IS_AUDIO_DISC (self->priv->current))
-		return;
-
-	session = BRASERO_BURN_SESSION (brasero_session_cfg_new ());
-	brasero_disc_set_session_param (BRASERO_DISC (self->priv->current), session);
-	brasero_disc_set_session_contents (BRASERO_DISC (self->priv->current), session);
-	brasero_jacket_edit_set_audio_tracks (BRASERO_JACKET_EDIT (cover),
-					      gtk_entry_get_text (GTK_ENTRY (self->priv->name_display)),
-					      self->priv->cover,
-					      brasero_burn_session_get_tracks (session));
-	g_object_unref (session);
-}
-
 /******************************** burning **************************************/
 static void
 brasero_project_no_song_dialog (BraseroProject *project)
@@ -901,10 +1033,31 @@ brasero_project_no_file_dialog (BraseroProject *project)
 			   GTK_MESSAGE_WARNING);
 }
 
+static void
+brasero_project_setup_session (BraseroProject *project,
+			       BraseroBurnSession *session)
+{
+	const gchar *label;
+
+	label = gtk_entry_get_text (GTK_ENTRY (project->priv->name_display));
+	brasero_burn_session_set_label (session, label);
+
+	if (project->priv->cover) {
+		GValue *value;
+
+		value = g_new0 (GValue, 1);
+		g_value_init (value, G_TYPE_STRING);
+		g_value_set_string (value, project->priv->cover);
+		brasero_burn_session_tag_add (session,
+					      BRASERO_COVER_URI,
+					      value);
+	}
+}
+
 void
 brasero_project_burn (BraseroProject *project)
 {
-	BraseroBurnSession *session;
+	BraseroSessionCfg *session;
 	BraseroDiscResult result;
 	GtkWidget *dialog;
 	gboolean success;
@@ -933,58 +1086,26 @@ brasero_project_burn (BraseroProject *project)
 	/* This is to stop the preview widget from playing */
 	brasero_uri_container_uri_selected (BRASERO_URI_CONTAINER (project));
 
-	/* setup, show, and run options dialog */
-	dialog = brasero_disc_option_dialog_new ();
-
-	if (!brasero_app_is_running (brasero_app_get_default ())) {
-		const gchar *label;
-
-		/* Set the label for the session in the burn option dialog but
-		 * only if we were started to burn a project (like through
-		 * nautilus). */
-		label = gtk_entry_get_text (GTK_ENTRY (project->priv->name_display));
-
-		session = brasero_burn_options_get_session (BRASERO_BURN_OPTIONS (dialog));
-		brasero_burn_session_set_label (session, label);
-		g_object_unref (session);
-	}
-
-	brasero_disc_option_dialog_set_disc (BRASERO_DISC_OPTION_DIALOG (dialog),
-					     project->priv->current);
+  	/* setup, show, and run options dialog */
+ 	session = brasero_session_cfg_new ();
+ 	brasero_disc_set_session_contents (project->priv->current, BRASERO_BURN_SESSION (session));
+ 	dialog = brasero_burn_options_new (session);
 
 	brasero_app_set_toplevel (brasero_app_get_default (), GTK_WINDOW (dialog));
 
 	result = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (result != GTK_RESPONSE_OK) {
+		g_object_unref (session);
 		gtk_widget_destroy (dialog);
 		goto end;
 	}
 
-	session = brasero_disc_option_dialog_get_session (BRASERO_DISC_OPTION_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
-	if (brasero_app_is_running (brasero_app_get_default ())) {
-		const gchar *label;
-
-		/* Set the label for the session but only if we weren't started
-		 * to burn a project (like through nautilus). */
-		label = gtk_entry_get_text (GTK_ENTRY (project->priv->name_display));
-		brasero_burn_session_set_label (session, label);
-	}
-
-	if (project->priv->cover) {
-		GValue *value;
-
-		value = g_new0 (GValue, 1);
-		g_value_init (value, G_TYPE_STRING);
-		g_value_set_string (value, project->priv->cover);
-		brasero_burn_session_tag_add (session,
-					      BRASERO_COVER_URI,
-					      value);
-	}
+	brasero_project_setup_session (project, BRASERO_BURN_SESSION (session));
 
 	/* now setup the burn dialog */
-	success = brasero_app_burn (brasero_app_get_default (), session);
+	success = brasero_app_burn (brasero_app_get_default (), BRASERO_BURN_SESSION (session));
 
     	project->priv->burnt = success;
 	g_object_unref (session);
@@ -994,6 +1115,28 @@ end:
 	project->priv->is_burning = 0;
 }
 
+/******************************** cover ****************************************/
+void
+brasero_project_create_audio_cover (BraseroProject *project,
+				    BraseroJacketEdit *cover)
+{
+	BraseroBurnSession *session;
+	GtkWidget *window;
+
+	if (!BRASERO_IS_AUDIO_DISC (project->priv->current))
+		return;
+
+	session = BRASERO_BURN_SESSION (brasero_session_cfg_new ());
+	brasero_disc_set_session_contents (BRASERO_DISC (project->priv->current), session);
+	brasero_project_setup_session (project, session);
+
+	window = brasero_session_edit_cover (session, gtk_widget_get_toplevel (GTK_WIDGET (project)));
+	g_object_unref (session);
+
+	gtk_dialog_run (GTK_DIALOG (window));
+	gtk_widget_destroy (window);
+}
+
 /********************************     ******************************************/
 static void
 brasero_project_switch (BraseroProject *project, BraseroProjectType type)
@@ -1008,6 +1151,10 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 		project->priv->project_status = NULL;
 	}
 
+	gtk_image_set_from_icon_name (GTK_IMAGE (project->priv->icon_img),
+				      "media-optical",
+				      GTK_ICON_SIZE_LARGE_TOOLBAR);
+
 	if (project->priv->current)
 		brasero_disc_reset (project->priv->current);
 
@@ -1039,6 +1186,7 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 					  project->priv->merge_id);
 
 	if (type == BRASERO_PROJECT_TYPE_AUDIO) {
+		gtk_widget_hide (project->priv->button_img);
 		project->priv->current = BRASERO_DISC (project->priv->audio);
 		project->priv->merge_id = brasero_disc_add_ui (project->priv->current,
 							       project->priv->manager,
@@ -1048,6 +1196,7 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 		brasero_project_update_project_size (project, 0);
 	}
 	else if (type == BRASERO_PROJECT_TYPE_DATA) {
+		gtk_widget_show (project->priv->button_img);
 		project->priv->current = BRASERO_DISC (project->priv->data);
 		project->priv->merge_id = brasero_disc_add_ui (project->priv->current,
 							       project->priv->manager,
@@ -1057,6 +1206,7 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 		brasero_project_update_project_size (project, 0);
 	}
 	else if (type == BRASERO_PROJECT_TYPE_VIDEO) {
+		gtk_widget_hide (project->priv->button_img);
 		project->priv->current = BRASERO_DISC (project->priv->video);
 		project->priv->merge_id = brasero_disc_add_ui (project->priv->current,
 							       project->priv->manager,
@@ -1476,7 +1626,7 @@ brasero_project_add_uris_cb (GtkAction *action,
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (project->priv->chooser), filter);
 
 	filter = gtk_file_filter_new ();
-	gtk_file_filter_set_name (filter, _("Audio files only"));
+	gtk_file_filter_set_name (filter, _("Audio files"));
 	gtk_file_filter_add_mime_type (filter, "audio/*");
 	gtk_file_filter_add_mime_type (filter, "application/ogg");
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (project->priv->chooser), filter);
@@ -1485,7 +1635,7 @@ brasero_project_add_uris_cb (GtkAction *action,
 		gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (project->priv->chooser), filter);
 
 	filter = gtk_file_filter_new ();
-	gtk_file_filter_set_name (filter, _("Movies only"));
+	gtk_file_filter_set_name (filter, _("Movies"));
 	gtk_file_filter_add_mime_type (filter, "video/*");
 	gtk_file_filter_add_mime_type (filter, "application/ogg");
 	gtk_file_filter_add_mime_type (filter, "application/x-flash-video");
@@ -1493,7 +1643,7 @@ brasero_project_add_uris_cb (GtkAction *action,
 
 	filter = gtk_file_filter_new ();
 	/* Translators: this is an image, a picture, not a "Disc Image" */
-	gtk_file_filter_set_name (filter, _("Image files only"));
+	gtk_file_filter_set_name (filter, C_("picture", "Image files"));
 	gtk_file_filter_add_mime_type (filter, "image/*");
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (project->priv->chooser), filter);
 
@@ -1682,10 +1832,13 @@ brasero_project_set_uri (BraseroProject *project,
 	/* update the name of the main window */
     	BRASERO_GET_BASENAME_FOR_DISPLAY (uri, name);
 	if (type == BRASERO_PROJECT_TYPE_DATA)
+		/* Translators: %s is the name of the project */
 		title = g_strdup_printf (_("Brasero - %s (Data Disc)"), name);
 	else if (type == BRASERO_PROJECT_TYPE_AUDIO)
+		/* Translators: %s is the name of the project */
 		title = g_strdup_printf (_("Brasero - %s (Audio Disc)"), name);
 	else if (type == BRASERO_PROJECT_TYPE_VIDEO)
+		/* Translators: %s is the name of the project */
 		title = g_strdup_printf (_("Brasero - %s (Video Disc)"), name);
 	else
 		title = NULL;
@@ -1855,7 +2008,7 @@ _save_audio_track_xml (xmlTextWriter *project,
 
 	for (iter = track->contents.tracks; iter; iter = iter->next) {
 		BraseroDiscSong *song;
-		BraseroSongInfo *info;
+		BraseroStreamInfo *info;
 		xmlChar *escaped;
 		gchar *start;
 		gchar *isrc;
@@ -1969,6 +2122,13 @@ _save_data_track_xml (xmlTextWriter *project,
 	GSList *grafts;
 	BraseroGraftPt *graft;
 
+	if (track->contents.data.icon) {
+		/* Write the icon if any */
+		success = xmlTextWriterWriteElement (project, (xmlChar *) "icon", (xmlChar *) track->contents.data.icon);
+		if (success < 0)
+			return FALSE;
+	}
+
 	for (grafts = track->contents.data.grafts; grafts; grafts = grafts->next) {
 		graft = grafts->data;
 
@@ -2193,7 +2353,7 @@ brasero_project_save_audio_project_plain_text (BraseroProject *proj,
 
 	for (iter = track->contents.tracks; iter; iter = iter->next) {
 		BraseroDiscSong *song;
-		BraseroSongInfo *info;
+		BraseroStreamInfo *info;
 		gchar *time;
 
 		song = iter->data;
@@ -2203,7 +2363,7 @@ brasero_project_save_audio_project_plain_text (BraseroProject *proj,
 		if (written != strlen (info->title))
 			goto error;
 
-		time = brasero_utils_get_time_string (song->end - song->start, TRUE, FALSE);
+		time = brasero_units_get_time_string (song->end - song->start, TRUE, FALSE);
 		if (time) {
 			written = fwrite ("\t", 1, 1, file);
 			if (written != 1)
@@ -2305,7 +2465,7 @@ brasero_project_save_audio_project_playlist (BraseroProject *proj,
 	model = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
 	for (iter = track->contents.tracks; iter; iter = iter->next) {
 		BraseroDiscSong *song;
-		BraseroSongInfo *info;
+		BraseroStreamInfo *info;
 
 		song = iter->data;
 		info = song->info;
@@ -2374,6 +2534,7 @@ brasero_project_save_project_real (BraseroProject *project,
 		return FALSE;
 
 	bzero (&track, sizeof (track));
+
 	result = brasero_disc_get_track (project->priv->current, &track);
 	if (result == BRASERO_DISC_ERROR_EMPTY_SELECTION) {
 		if (BRASERO_IS_AUDIO_DISC (project->priv->current))
@@ -2626,6 +2787,8 @@ brasero_project_save_session (BraseroProject *project,
 
     	bzero (&track, sizeof (track));
 	if (brasero_disc_get_track (project->priv->current, &track) == BRASERO_DISC_OK) {
+		/* NOTE: is this right? because brasero could not shut itself
+		 * down if an error occurs. */
 		if (!brasero_project_save_project_xml (project,
 						       uri,
 						       &track,
